//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#define MAGICMA  20050610

extern double Lots = 0.1;
extern string A0 = " ";
extern string A1 = "  . -";
extern string A2 = "   Lots  ";
extern bool MaximumRisk_DecreaseFactor = true; //      -     
                                  //(    "0")         
extern double MaximumRisk = 0.02;
extern double DecreaseFactor = 3;

extern string A3 = "   ."; //       150 (   )
extern string A4 = "   f"; //    
extern bool optimal_f = true;           //         .: /
extern double Transaction_number = 150; //  ,         
                                        //  f.    -  .
extern string A5 = "  ";
extern double MovingPeriod       = 12;
extern double MovingShift        = 6;

//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot;  
   string Symb   =Symbol();                    //  .
   double One_Lot=MarketInfo(Symb,MODE_MARGINREQUIRED);//. 1 
   double Min_Lot=MarketInfo(Symb,MODE_MINLOT);// . . 
   double Step   =MarketInfo(Symb,MODE_LOTSTEP);//  
   double Free   =AccountFreeMargin();         //  
   
//--------------------------------------------------------------------
   if (Lots>0)                                 //   ..
      {                                         // .. 
       double Money=Lots*One_Lot;               //  
       if(Money<=AccountFreeMargin())           //  ..
         lot=Lots;                             // .. 
       else                                     //   ..
         lot=MathFloor(Free/One_Lot/Step)*Step;//  
      }
   
   else  {                                        //    
      
      if (MaximumRisk_DecreaseFactor)             
        {                                         // ..  
         if (MaximumRisk > 100)                   //   ..
            MaximumRisk=100;                      // ..    100
         if (MaximumRisk==0)                      //   0 ..
             {
              lot=Min_Lot;                          // ..  
              return(lot);
             } 
         else                                     // . .:
           { 
            int    orders=HistoryTotal();     // history orders total
            int    losses=0;                  // number of losses orders without a break
            lot=MathFloor(Free*MaximumRisk/100/One_Lot/Step)*Step;//
            //---- calcuulate number of losses orders without a break
            if(DecreaseFactor>0)
              {
               for(int i=orders-1;i>=0;i--)
                  {
                   if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
                   if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
                   //----
                   if(OrderProfit()>0) break;
                   if(OrderProfit()<0) losses++;
                  }
               if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
              }    
             //return(lot);      
            } 
         }
          
     if (optimal_f) //---   f ---
        {   
          
          double Mas_Outcome_of_transactions [10000];        //  /  
          int Qnt = 0, Orders = OrdersHistoryTotal();        //      
          ArrayInitialize(Mas_Outcome_of_transactions, 0);   //  
          double f, D, SUMM, TWR, G, G_Rez, H,A,B, Pow;
          int orderIndex;
   
          for (orderIndex = 0; orderIndex < Orders; orderIndex++)
          {   
             if (!OrderSelect(orderIndex, SELECT_BY_POS, MODE_HISTORY))
              {
                Print("      (",GetLastError(),")");
                continue;
              }
   
             if (OrderSymbol() != Symbol() || OrderMagicNumber() != MAGICMA || OrderCloseTime()==0)
                continue; 
         
             int lastType = OrderType();
             double lastLots = OrderLots();
             double lastProfit = OrderProfit() + OrderSwap();
      
             if (orderIndex == 0 || lastProfit < D)
                D = lastProfit;
      
             Mas_Outcome_of_transactions[Qnt] = lastProfit;  //   /     
             SUMM=SUMM+lastProfit;
             Qnt++;                                          //        
          }   
   
         if (Transaction_number<Qnt) { //  -       - f 
            Pow = 1/NormalizeDouble(Orders, 0);
            for (f = 0.01; f<=1.0; f=f+0.01)                   //    f     ,
            {                                                  //   TWR - 
               G= 1;
               for ( orderIndex = 1; orderIndex < Qnt; orderIndex++) //   f     
                {                                                     //      TWR
                  TWR = 1+f*(-Mas_Outcome_of_transactions[orderIndex]/(D));
                  G *= NormalizeDouble (MathPow(TWR, Pow),8);
                }
               if (G > G_Rez)  G_Rez = G;//   > ,     
               else break;               //        f
            }
               
            
            if (f>0) H=D/(-f); //   ( f)   0,1 .
            lot = NormalizeDouble((AccountFreeMargin()/H)*Min_Lot,1);
            if (lot==0)    lot=Min_Lot;
            Print("H=D/(-f): ", H, " lot = ", DoubleToStr (lot,1), "Transaction_number = ", Transaction_number);  
            Print("G_Rez  = ", DoubleToStr (G_Rez,8), "  f = ", f);             
            Print("    , D = ", DoubleToStr(D, 2), " Pow (1/Orders)= ", DoubleToStr(Pow, 8));
            Print("  = ",   Qnt,
                "  / = ", SUMM,
                "   ",         Qnt,
                "   / = ", Mas_Outcome_of_transactions[Qnt-1]);  
                
            return(lot);         
          }    //    if (Transaction_number<Qnt)                   
          else {
             lot=Min_Lot; 
             Print("  = ",   Qnt, " Transaction_number = ", Transaction_number);
             return(lot); 
          } 
  
      }  //    f     
  
  
  
   }//   else - MaximumRisk_DecreaseFactor  optimal_f
         
   
   
//----------------------------------------------------------------------------------------------------------     
  if (lot < Min_Lot)                      //   ..
      lot=Min_Lot;                        // ..  
   if (lot*One_Lot > AccountFreeMargin()) //   ..
     {                                    // .. . :(
      Print("    . : ", AccountFreeMargin(), " <", lot*One_Lot); // ..
      return(0);                      // ..  
     }    
//---- return lot size
   //if(lot<0.1) lot=0.1;
   
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double ma;
   int    res;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
//---- sell conditions
   if(Open[1]>ma && Close[1]<ma)  
     {
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,0,0,"",MAGICMA,0,Red);
      return;
     }
//---- buy conditions
   if(Open[1]<ma && Close[1]>ma)  
     {
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,0,0,"",MAGICMA,0,Blue);
      return;
     }
//----
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double ma;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
         if(Open[1]>ma && Close[1]<ma) OrderClose(OrderTicket(),OrderLots(),Bid,3,White);
         break;
        }
      if(OrderType()==OP_SELL)
        {
         if(Open[1]<ma && Close[1]>ma) OrderClose(OrderTicket(),OrderLots(),Ask,3,White);
         break;
        }
     }
//----
  }
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
   else                                    CheckForClose();
//----
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
  

  return(0);
  }

